use std::collections::HashMap;

use enum_map::EnumMap;
use serde::Deserialize;

use crate::model::icon_descriptor::IconDescriptor;
use crate::model::position::KnobPosition;
use crate::model::rgb::RGB8WithOptionalA;
use crate::modes;

#[derive(Debug, Deserialize)]
pub struct File {
    pub id: Option<String>,
    pub knobs: HashMap<KnobPosition, Knob>,
}

#[derive(Debug)]
pub struct Page {
    pub id: String,
    pub knobs: EnumMap<KnobPosition, Knob>,
}

#[derive(Debug, Default, Deserialize)]
pub struct Knob {
    #[serde(default, flatten)]
    pub base_style: KnobStyle,
    #[serde(default)]
    pub mode: KnobModes,
}

#[derive(Debug, Default, Clone, Deserialize)]
pub struct KnobIndicators {
    pub bar: Option<KnobIndicatorBarConfig>,
    pub circle: Option<KnobIndicatorCircleConfig>,
}

#[derive(Debug, Clone, Deserialize)]
pub struct KnobIndicatorBarConfig {
    pub color: Option<RGB8WithOptionalA>,
}

#[derive(Debug, Clone, Deserialize)]
pub struct KnobIndicatorCircleConfig {
    pub color: Option<RGB8WithOptionalA>,
    pub width: Option<u8>,
    pub radius: Option<u8>,
}

#[derive(Debug, Default, Clone, Deserialize)]
pub struct KnobStyle {
    pub label: Option<String>,
    pub icon: Option<IconDescriptor>,
    pub indicators: Option<KnobIndicators>,
}

#[derive(Debug, Default, Deserialize)]
pub struct KnobModes {
    pub audio_volume: Option<modes::knob::audio_volume::Config>,
}

pub type StyleByStateMap<State> = HashMap<State, KnobStyle>;
