use std::sync::Arc;

use serde::{Deserialize, Serialize};
use tokio::sync::broadcast;

use crate::model;
use crate::model::position::KeyPath;
use crate::runner::state::StateChangeCommand;

pub mod home_assistant;
pub mod media;
pub mod spotify;
pub mod timer;
pub mod vibrate;

#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
pub enum KeyEvent {
    Press,
    VisibilityChange { is_visible: bool },
}

pub fn start_handlers(
    keys: impl Iterator<Item = (KeyPath, Arc<model::key_page::KeyConfig>)>,
    events: broadcast::Sender<(KeyPath, KeyEvent)>,
    commands: flume::Sender<StateChangeCommand>,
) {
    for (path, config) in keys {
        let mut events = events.subscribe();
        let own_events = broadcast::Sender::new(5);

        if let Some(c) = &config.mode.media__play_pause {
            tokio::spawn(media::handle(path.clone(), Arc::clone(c), own_events.subscribe(), commands.clone()));
        }

        tokio::spawn(async move {
            while let Ok((p, e)) = events.recv().await {
                #[allow(clippy::collapsible_if)]
                if p == path {
                    if own_events.send(e).is_err() {
                        break;
                    }
                }
            }
        });
    }
}
